@@ -16,12 +16,13 @@ import (
 	"time"
 
 	"github.com/AlecAivazis/survey/v2"
+	"github.com/agnivade/levenshtein"
 	"github.com/urfave/cli/v2"
 	"gopkg.in/yaml.v3"
 )
 
 const (
-	Version               = "v0.1.5"
+	Version               = "v0.1.13"
 	GithubRepo            = "https://github.com/eduardoagarcia/shef"
 	PublicRecipesFilename = "recipes.tar.gz"
 	PublicRecipesFolder   = "recipes"
@@ -46,16 +47,20 @@ type Operation struct {
 	ControlFlow   interface{} `yaml:"control_flow,omitempty"`
 	Operations    []Operation `yaml:"operations,omitempty"`
 	ExecutionMode string      `yaml:"execution_mode,omitempty"`
+	OutputFormat  string      `yaml:"output_format,omitempty"`
 	Silent        bool        `yaml:"silent,omitempty"`
 	Condition     string      `yaml:"condition,omitempty"`
 	OnSuccess     string      `yaml:"on_success,omitempty"`
 	OnFailure     string      `yaml:"on_failure,omitempty"`
 	Transform     string      `yaml:"transform,omitempty"`
 	Prompts       []Prompt    `yaml:"prompts,omitempty"`
+	Break         bool        `yaml:"break,omitempty"`
+	Exit          bool        `yaml:"exit,omitempty"`
 }
 
 type Prompt struct {
 	Name            string            `yaml:"name"`
+	ID              string            `yaml:"id,omitempty"`
 	Type            string            `yaml:"type"`
 	Message         string            `yaml:"message"`
 	Default         string            `yaml:"default,omitempty"`
@@ -104,6 +109,38 @@ func (ctx *ExecutionContext) templateVars() map[string]interface{} {
 	return vars
 }
 
+type Color string
+
+type Style string
+
+const (
+	ColorNone      Color = ""
+	ColorBlack     Color = "black"
+	ColorRed       Color = "red"
+	ColorGreen     Color = "green"
+	ColorYellow    Color = "yellow"
+	ColorBlue      Color = "blue"
+	ColorMagenta   Color = "magenta"
+	ColorCyan      Color = "cyan"
+	ColorWhite     Color = "white"
+	BgColorBlack   Color = "bg-black"
+	BgColorRed     Color = "bg-red"
+	BgColorGreen   Color = "bg-green"
+	BgColorYellow  Color = "bg-yellow"
+	BgColorBlue    Color = "bg-blue"
+	BgColorMagenta Color = "bg-magenta"
+	BgColorCyan    Color = "bg-cyan"
+	BgColorWhite   Color = "bg-white"
+)
+
+const (
+	StyleNone      Style = ""
+	StyleBold      Style = "bold"
+	StyleDim       Style = "dim"
+	StyleItalic    Style = "italic"
+	StyleUnderline Style = "underline"
+)
+
 var colorCodes = map[string]string{
 	"black":      "\033[30m",
 	"red":        "\033[31m",
@@ -279,6 +316,7 @@ func loadConfig(filename string) (*Config, error) {
 
 func loadRecipes(sources []string, category string) ([]Recipe, error) {
 	var allRecipes []Recipe
+	lowerCategory := strings.ToLower(category)
 
 	for _, source := range sources {
 		config, err := loadConfig(source)
@@ -293,7 +331,7 @@ func loadRecipes(sources []string, category string) ([]Recipe, error) {
 		}
 
 		for _, recipe := range config.Recipes {
-			if recipe.Category == category {
+			if strings.ToLower(recipe.Category) == lowerCategory {
 				allRecipes = append(allRecipes, recipe)
 			}
 		}
@@ -358,16 +396,31 @@ func findRecipeSourcesByType(localDir, userDir, publicRepo bool) ([]string, erro
 		}
 	}
 
-	if userDir || publicRepo {
-		homeDir, err := os.UserHomeDir()
-		if err == nil {
-			userRoot := filepath.Join(homeDir, ".shef")
-			if _, err := os.Stat(userRoot); err == nil {
-				if userFiles, err := findYamlFiles(userRoot); err == nil {
+	homeDir, err := os.UserHomeDir()
+	if err == nil {
+		userRoot := filepath.Join(homeDir, ".shef")
+
+		if userDir {
+			userSpecificDir := filepath.Join(userRoot, "user")
+			if _, err := os.Stat(userSpecificDir); err == nil {
+				if userFiles, err := findYamlFiles(userSpecificDir); err == nil {
 					sources = append(sources, userFiles...)
 				}
 			}
 		}
+
+		if publicRepo {
+			publicSpecificDir := filepath.Join(userRoot, "public")
+			if _, err := os.Stat(publicSpecificDir); err == nil {
+				if publicFiles, err := findYamlFiles(publicSpecificDir); err == nil {
+					sources = append(sources, publicFiles...)
+				}
+			}
+		}
+
+		if isLinux() && (userDir || publicRepo) {
+			sources = addXDGRecipeSources(sources, userDir, publicRepo, findYamlFiles)
+		}
 	}
 
 	return sources, nil
@@ -714,7 +767,7 @@ func parseOptionsFromOutput(output string) []string {
 	return result
 }
 
-func executeCommand(cmdStr string, input string, executionMode string) (string, error) {
+func executeCommand(cmdStr string, input string, executionMode string, outputFormat string) (string, error) {
 	if executionMode == "" {
 		executionMode = "standard"
 	}
@@ -735,7 +788,22 @@ func executeCommand(cmdStr string, input string, executionMode string) (string,
 			return "", fmt.Errorf("command failed: %w\nStderr: %s", err, stderr.String())
 		}
 
-		return strings.TrimSpace(stdout.String()), nil
+		switch outputFormat {
+		case "trim":
+			return strings.TrimSpace(stdout.String()), nil
+		case "lines":
+			var lines []string
+			for _, line := range strings.Split(stdout.String(), "\n") {
+				if trimmedLine := strings.TrimSpace(line); trimmedLine != "" {
+					lines = append(lines, trimmedLine)
+				}
+			}
+			return strings.Join(lines, "\n"), nil
+		case "raw", "":
+			return stdout.String(), nil
+		default:
+			return stdout.String(), nil
+		}
 	}
 
 	// Execute command with interactive or streaming mode
@@ -1092,21 +1160,56 @@ func executeRecipe(recipe Recipe, input string, vars map[string]interface{}, deb
 		}
 	}
 
-	var executeOp func(op Operation, depth int) error
-	executeOp = func(op Operation, depth int) error {
+	var executeOp func(op Operation, depth int) (bool, error)
+	executeOp = func(op Operation, depth int) (bool, error) {
 		if depth > 50 {
-			return fmt.Errorf("possible infinite loop detected (max depth reached)")
+			return false, fmt.Errorf("possible infinite loop detected (max depth reached)")
 		}
 
+		// 1. Check the condition first
+		if op.Condition != "" {
+			if debug {
+				fmt.Printf("Evaluating condition: %s\n", op.Condition)
+			}
+			result, err := evaluateCondition(op.Condition, &ctx)
+			if err != nil {
+				return false, fmt.Errorf("condition evaluation failed: %w", err)
+			}
+
+			if !result {
+				if debug {
+					fmt.Printf("Skipping operation '%s' (condition not met)\n", op.Name)
+				}
+				return false, nil
+			}
+		}
+
+		// 2. Run the prompts
+		for _, prompt := range op.Prompts {
+			value, err := handlePrompt(prompt, &ctx)
+			if err != nil {
+				return false, err
+			}
+
+			varName := prompt.Name
+			if prompt.ID != "" {
+				varName = prompt.ID
+			}
+			ctx.Vars[varName] = value
+		}
+
+		// 3. Run the control flow if it exists
+		var controlFlowExit bool
+		var controlFlowErr error
 		if op.ControlFlow != nil {
 			flowMap, ok := op.ControlFlow.(map[string]interface{})
 			if !ok {
-				return fmt.Errorf("invalid control_flow structure")
+				return false, fmt.Errorf("invalid control_flow structure")
 			}
 
 			typeVal, ok := flowMap["type"].(string)
 			if !ok {
-				return fmt.Errorf("control_flow requires a 'type' field")
+				return false, fmt.Errorf("control_flow requires a 'type' field")
 			}
 
 			switch typeVal {
@@ -1114,65 +1217,53 @@ func executeRecipe(recipe Recipe, input string, vars map[string]interface{}, deb
 			case "foreach":
 				forEach, err := op.GetForEachFlow()
 				if err != nil {
-					return err
+					return false, err
 				}
-				return ExecuteForEach(op, forEach, &ctx, depth, executeOp, debug)
+				controlFlowExit, controlFlowErr = ExecuteForEach(op, forEach, &ctx, depth, executeOp, debug)
 
 			case "while":
 				whileFlow, err := op.GetWhileFlow()
 				if err != nil {
-					return err
+					return false, err
 				}
-				return ExecuteWhile(op, whileFlow, &ctx, depth, executeOp, debug)
+				controlFlowExit, controlFlowErr = ExecuteWhile(op, whileFlow, &ctx, depth, executeOp, debug)
 
 			case "for":
 				forFlow, err := op.GetForFlow()
 				if err != nil {
-					return err
+					return false, err
 				}
-				return ExecuteFor(op, forFlow, &ctx, depth, executeOp, debug)
+				controlFlowExit, controlFlowErr = ExecuteFor(op, forFlow, &ctx, depth, executeOp, debug)
 
 			default:
-				return fmt.Errorf("unknown control_flow type: %s", typeVal)
+				return false, fmt.Errorf("unknown control_flow type: %s", typeVal)
 			}
 		}
 
-		if op.Condition != "" {
+		if controlFlowErr != nil {
+			return op.Exit, controlFlowErr
+		}
+
+		if controlFlowExit {
 			if debug {
-				fmt.Printf("Evaluating condition: %s\n", op.Condition)
-			}
-			result, err := evaluateCondition(op.Condition, &ctx)
-			if err != nil {
-				return fmt.Errorf("condition evaluation failed: %w", err)
-			}
-
-			if !result {
-				if debug {
-					fmt.Printf("Skipping operation '%s' (condition not met)\n", op.Name)
-				}
-
-				return nil
+				fmt.Printf("Exiting recipe due to exit flag inside for control flow\n")
 			}
+			return true, nil
 		}
 
-		for _, prompt := range op.Prompts {
-			value, err := handlePrompt(prompt, &ctx)
-			if err != nil {
-				return err
-			}
-			ctx.Vars[prompt.Name] = value
-		}
-
+		// 4. Run the command
 		cmd, err := renderTemplate(op.Command, ctx.templateVars())
 		if err != nil {
-			return fmt.Errorf("failed to render command template: %w", err)
+			return false, fmt.Errorf("failed to render command template: %w", err)
 		}
 
 		if debug {
 			fmt.Printf("Running command: %s\n", cmd)
 		}
 
-		output, err := executeCommand(cmd, ctx.Data, op.ExecutionMode)
+		ctx.Vars["error"] = ""
+
+		output, err := executeCommand(cmd, ctx.Data, op.ExecutionMode, op.OutputFormat)
 		operationSuccess := err == nil
 
 		if op.ID != "" {
@@ -1180,6 +1271,8 @@ func executeRecipe(recipe Recipe, input string, vars map[string]interface{}, deb
 		}
 
 		if err != nil {
+			ctx.Vars["error"] = err.Error()
+
 			if debug {
 				fmt.Printf("Warning: command execution had errors: %v\n", err)
 			}
@@ -1191,25 +1284,29 @@ func executeRecipe(recipe Recipe, input string, vars map[string]interface{}, deb
 
 				nextOp, exists := opMap[op.OnFailure]
 				if !exists {
-					return fmt.Errorf("on_failure operation %s not found", op.OnFailure)
+					return false, fmt.Errorf("on_failure operation %s not found", op.OnFailure)
 				}
-				return executeOp(nextOp, depth+1)
+				shouldExit, err := executeOp(nextOp, depth+1)
+				return shouldExit || op.Exit, err
 			}
 
+			fmt.Printf("Error in operation '%s': \n%v\n", op.Name, err)
+
 			var continueExecution bool
 			prompt := &survey.Confirm{
-				Message: "Command had errors. Continue with recipe execution?",
+				Message: "Continue with recipe execution?",
 				Default: false,
 			}
 			if err := survey.AskOne(prompt, &continueExecution); err != nil {
-				return err
+				return false, err
 			}
 
 			if !continueExecution {
-				return fmt.Errorf("recipe execution aborted by user after command error")
+				return true, fmt.Errorf("recipe execution aborted by user after command error")
 			}
 		}
 
+		// 5. Run the transforms
 		if op.Transform != "" {
 			transformedOutput, err := transformOutput(output, op.Transform, &ctx)
 			if err != nil {
@@ -1231,21 +1328,29 @@ func executeRecipe(recipe Recipe, input string, vars map[string]interface{}, deb
 			fmt.Println(output)
 		}
 
+		// 6. Run the on_success handler
 		if op.OnSuccess != "" && operationSuccess {
 			nextOp, exists := opMap[op.OnSuccess]
 			if !exists {
-				return fmt.Errorf("on_success operation %s not found", op.OnSuccess)
+				return false, fmt.Errorf("on_success operation %s not found", op.OnSuccess)
 			}
-			return executeOp(nextOp, depth+1)
+			shouldExit, err := executeOp(nextOp, depth+1)
+			return shouldExit || op.Exit, err
 		}
 
 		if debug {
 			fmt.Printf("Operation %s result: %v\n", op.ID, ctx.OperationResults[op.ID])
 			fmt.Printf("Handler for on_success: '%s'\n", op.OnSuccess)
 			fmt.Printf("Handler for on_failure: '%s'\n", op.OnFailure)
+			if op.Exit {
+				fmt.Printf("Exit flag is set. Will exit after this operation.\n")
+			}
+			if op.Break {
+				fmt.Printf("Break flag is set. Will break out of control flow.\n")
+			}
 		}
 
-		return nil
+		return op.Exit, nil
 	}
 
 	for i, op := range recipe.Operations {
@@ -1260,9 +1365,17 @@ func executeRecipe(recipe Recipe, input string, vars map[string]interface{}, deb
 			fmt.Printf("Executing operation %d: %s\n", i+1, op.Name)
 		}
 
-		if err := executeOp(op, 0); err != nil {
+		shouldExit, err := executeOp(op, 0)
+		if err != nil {
 			return err
 		}
+
+		if shouldExit {
+			if debug {
+				fmt.Printf("Exiting recipe execution after operation: %s\n", op.Name)
+			}
+			return nil
+		}
 	}
 
 	return nil
@@ -1297,11 +1410,11 @@ func identifyHandlers(operations []Operation, handlerIDs map[string]bool) {
 
 func listRecipes(recipes []Recipe) {
 	if len(recipes) == 0 {
-		fmt.Println("No recipes found.")
+		fmt.Println(FormatText("No recipes found.", ColorYellow, StyleNone))
 		return
 	}
 
-	fmt.Println("Available recipes:")
+	fmt.Println("\nAvailable recipes:")
 
 	categories := make(map[string][]Recipe)
 	for _, recipe := range recipes {
@@ -1325,16 +1438,29 @@ func listRecipes(recipes []Recipe) {
 			return catRecipes[i].Name < catRecipes[j].Name
 		})
 
-		fmt.Printf("\n[%s]\n", category)
-		for i, recipe := range catRecipes {
-			fmt.Printf("%d. %s: %s\n", i+1, recipe.Name, recipe.Description)
+		fmt.Printf(
+			"\n  %s%s%s\n",
+			FormatText("[", ColorNone, StyleDim),
+			FormatText(strings.ToLower(category), ColorMagenta, StyleNone),
+			FormatText("]", ColorNone, StyleDim),
+		)
+		for _, recipe := range catRecipes {
+			fmt.Printf(
+				"    %s %s: %s\n",
+				FormatText("•", ColorNone, StyleDim),
+				FormatText(strings.ToLower(recipe.Name), ColorGreen, StyleBold),
+				strings.ToLower(recipe.Description),
+			)
 		}
 	}
+
+	fmt.Printf("\n\n")
 }
 
 func findRecipeByName(recipes []Recipe, name string) (*Recipe, error) {
+	lowerName := strings.ToLower(name)
 	for _, recipe := range recipes {
-		if recipe.Name == name {
+		if strings.ToLower(recipe.Name) == lowerName {
 			return &recipe, nil
 		}
 	}
@@ -1342,9 +1468,11 @@ func findRecipeByName(recipes []Recipe, name string) (*Recipe, error) {
 }
 
 func main() {
+	log.SetFlags(0)
+
 	app := &cli.App{
 		Name:    "shef",
-		Usage:   "Shef is a powerful CLI tool that lets you combine shell commands into reusable recipes.",
+		Usage:   "Shef is a powerful CLI tool for cooking up shell recipes.",
 		Version: Version,
 		Flags: []cli.Flag{
 			&cli.BoolFlag{
@@ -1376,11 +1504,19 @@ func main() {
 			&cli.PathFlag{
 				Name:    "recipe-file",
 				Aliases: []string{"r"},
-				Usage:   "Path to the recipe file",
+				Usage:   "Path to the recipe file (note: additional recipe flags not supported)",
 			},
 		},
 		Action: func(c *cli.Context) error {
 			args := c.Args().Slice()
+			if len(args) == 0 && !c.IsSet("recipe-file") {
+				err := cli.ShowAppHelp(c)
+				if err != nil {
+					return err
+				}
+				return nil
+			}
+
 			sourcePriority := getSourcePriority(c)
 			return handleRunCommand(c, args, sourcePriority)
 		},
@@ -1390,6 +1526,21 @@ func main() {
 				Aliases: []string{"ls", "l"},
 				Usage:   "List available recipes",
 				Flags: []cli.Flag{
+					&cli.BoolFlag{
+						Name:    "local",
+						Aliases: []string{"l"},
+						Usage:   "Filter to local recipes only",
+					},
+					&cli.BoolFlag{
+						Name:    "user",
+						Aliases: []string{"u"},
+						Usage:   "Filter to user recipes only",
+					},
+					&cli.BoolFlag{
+						Name:    "public",
+						Aliases: []string{"p"},
+						Usage:   "Filter to public recipes only",
+					},
 					&cli.BoolFlag{
 						Name:    "json",
 						Aliases: []string{"j"},
@@ -1408,26 +1559,64 @@ func main() {
 				},
 			},
 			{
-				Name:  "sync",
-				Usage: "Sync public recipes locally",
+				Name:    "sync",
+				Aliases: []string{"s"},
+				Usage:   "Sync public recipes locally",
 				Action: func(c *cli.Context) error {
 					return syncPublicRecipes()
 				},
 			},
+			{
+				Name:    "which",
+				Aliases: []string{"w"},
+				Usage:   "Show the location of a recipe file",
+				Flags: []cli.Flag{
+					&cli.BoolFlag{
+						Name:    "local",
+						Aliases: []string{"L"},
+						Usage:   "Force local recipes first",
+					},
+					&cli.BoolFlag{
+						Name:    "user",
+						Aliases: []string{"U"},
+						Usage:   "Force user recipes first",
+					},
+					&cli.BoolFlag{
+						Name:    "public",
+						Aliases: []string{"P"},
+						Usage:   "Force public recipes first",
+					},
+				},
+				Action: func(c *cli.Context) error {
+					args := c.Args().Slice()
+					sourcePriority := getSourcePriority(c)
+					return handleWhichCommand(args, sourcePriority)
+				},
+			},
 		},
 	}
 
 	if err := app.Run(os.Args); err != nil {
-		log.Fatal(err)
+		errorText := strings.ToLower(err.Error())
+		formattedErr := fmt.Sprintf(
+			"%s: %s",
+			FormatText("Error", ColorRed, StyleBold),
+			errorText,
+		)
+		log.Fatal(formattedErr)
 	}
 }
 
 func getSourcePriority(c *cli.Context) []string {
-	if c.Bool("local") {
+	useLocal := c.Bool("local") || c.Bool("L")
+	useUser := c.Bool("user") || c.Bool("U")
+	usePublic := c.Bool("public") || c.Bool("P")
+
+	if useLocal {
 		return []string{"local", "user", "public"}
-	} else if c.Bool("user") {
+	} else if useUser {
 		return []string{"user", "local", "public"}
-	} else if c.Bool("public") {
+	} else if usePublic {
 		return []string{"public", "local", "user"}
 	}
 
@@ -1440,15 +1629,27 @@ func handleListCommand(c *cli.Context, args []string, sourcePriority []string) e
 		category = args[0]
 	}
 
+	useLocal := c.Bool("local") || c.Bool("L")
+	useUser := c.Bool("user") || c.Bool("U")
+	usePublic := c.Bool("public") || c.Bool("P")
+
+	if !useLocal && !useUser && !usePublic {
+		useLocal = true
+		useUser = true
+		usePublic = true
+	}
+
 	var allRecipes []Recipe
 	recipeMap := make(map[string]bool)
 
 	for _, source := range sourcePriority {
-		useLocal := source == "local"
-		useUser := source == "user"
-		usePublic := source == "public"
+		if (source == "local" && !useLocal) ||
+			(source == "user" && !useUser) ||
+			(source == "public" && !usePublic) {
+			continue
+		}
 
-		sources, _ := findRecipeSourcesByType(useLocal, useUser, usePublic)
+		sources, _ := findRecipeSourcesByType(source == "local", source == "user", source == "public")
 		recipes, _ := loadRecipes(sources, category)
 
 		for _, r := range recipes {
@@ -1459,6 +1660,16 @@ func handleListCommand(c *cli.Context, args []string, sourcePriority []string) e
 		}
 	}
 
+	if category == "" {
+		var filteredRecipes []Recipe
+		for _, recipe := range allRecipes {
+			if recipe.Category != "demo" {
+				filteredRecipes = append(filteredRecipes, recipe)
+			}
+		}
+		allRecipes = filteredRecipes
+	}
+
 	if len(allRecipes) == 0 {
 		if c.Bool("json") {
 			fmt.Println("[]")
@@ -1522,15 +1733,33 @@ func handleRunCommand(c *cli.Context, args []string, sourcePriority []string) er
 			return fmt.Errorf("no recipe specified. Use shef ls to list available recipes")
 		}
 
-		recipe, err := findRecipeInSources(args[0], "", sourcePriority)
+		var recipe *Recipe
+		var err error
 
-		if err != nil && len(args) > 1 {
-			recipe, err = findRecipeInSources(args[1], args[0], sourcePriority)
+		recipe, err = findRecipeInSources(args[0], "", sourcePriority, false)
+		if err == nil {
+			remainingArgs = args[1:]
+		} else if len(args) > 1 {
+			recipe, err = findRecipeInSources(args[1], args[0], sourcePriority, false)
 			if err == nil {
 				remainingArgs = args[2:]
 			}
-		} else if err == nil {
-			remainingArgs = args[1:]
+		}
+
+		if err != nil {
+			if len(args) > 1 {
+				recipe, err = findRecipeInSources(args[1], args[0], sourcePriority, true)
+				if err == nil {
+					remainingArgs = args[2:]
+				}
+			}
+
+			if err != nil {
+				recipe, err = findRecipeInSources(args[0], "", sourcePriority, true)
+				if err == nil {
+					remainingArgs = args[1:]
+				}
+			}
 		}
 
 		if err != nil {
@@ -1593,7 +1822,7 @@ func processRemainingArgs(args []string) (string, map[string]interface{}) {
 	return input, vars
 }
 
-func findRecipeInSources(recipeName, category string, sourcePriority []string) (*Recipe, error) {
+func findRecipeInSources(recipeName, category string, sourcePriority []string, fuzzyMatch bool) (*Recipe, error) {
 	for _, source := range sourcePriority {
 		useLocal := source == "local"
 		useUser := source == "user"
@@ -1616,5 +1845,177 @@ func findRecipeInSources(recipeName, category string, sourcePriority []string) (
 		}
 	}
 
+	if fuzzyMatch {
+		var allRecipes []Recipe
+		seenRecipeNames := make(map[string]bool)
+
+		for _, source := range sourcePriority {
+			useLocal := source == "local"
+			useUser := source == "user"
+			usePublic := source == "public"
+
+			sources, _ := findRecipeSourcesByType(useLocal, useUser, usePublic)
+			recipes, _ := loadRecipes(sources, "")
+
+			for _, recipe := range recipes {
+				if !seenRecipeNames[recipe.Name] {
+					allRecipes = append(allRecipes, recipe)
+					seenRecipeNames[recipe.Name] = true
+				}
+			}
+		}
+
+		if len(allRecipes) > 0 {
+			recipeNames := make([]string, 0, len(allRecipes))
+			recipeMap := make(map[string]Recipe)
+
+			for _, recipe := range allRecipes {
+				recipeNames = append(recipeNames, recipe.Name)
+				recipeMap[recipe.Name] = recipe
+			}
+
+			if match, found := fuzzyMatchRecipe(recipeName, recipeNames, recipeMap); found {
+				return match, nil
+			}
+		}
+	}
+
 	return nil, fmt.Errorf("recipe not found: %s", recipeName)
 }
+
+func fuzzyMatchRecipe(recipeName string, recipeNames []string, recipeMap map[string]Recipe) (*Recipe, bool) {
+	if len(recipeNames) == 0 {
+		return nil, false
+	}
+
+	type nameDistance struct {
+		name     string
+		distance int
+	}
+	var matches []nameDistance
+
+	for _, name := range recipeNames {
+		distance := levenshtein.ComputeDistance(recipeName, name)
+		matches = append(matches, nameDistance{name: name, distance: distance})
+	}
+
+	sort.Slice(matches, func(i, j int) bool {
+		return matches[i].distance < matches[j].distance
+	})
+
+	if len(matches) > 0 {
+		bestMatch := matches[0]
+		recipe := recipeMap[bestMatch.name]
+
+		var confirm bool
+		var promptMessage string
+
+		if recipe.Category != "" {
+			promptMessage = fmt.Sprintf("Did you mean [%s] '%s'?", recipe.Category, bestMatch.name)
+		} else {
+			promptMessage = fmt.Sprintf("Did you mean '%s'?", bestMatch.name)
+		}
+
+		prompt := &survey.Confirm{
+			Message: promptMessage,
+			Default: true,
+		}
+
+		if err := survey.AskOne(prompt, &confirm); err == nil && confirm {
+			return &recipe, true
+		}
+	}
+
+	return nil, false
+}
+
+func handleWhichCommand(args []string, sourcePriority []string) error {
+	if len(args) == 0 {
+		return fmt.Errorf("you must specify a recipe name")
+	}
+
+	var category string
+	var recipeName string
+
+	if len(args) >= 2 {
+		category = args[0]
+		recipeName = args[1]
+	} else {
+		recipeName = args[0]
+	}
+
+	sourcePath, err := findRecipeSourceFile(recipeName, category, sourcePriority)
+	if err != nil {
+		return err
+	}
+
+	fmt.Println(sourcePath)
+	return nil
+}
+
+func findRecipeSourceFile(recipeName, category string, sourcePriority []string) (string, error) {
+	for _, source := range sourcePriority {
+		useLocal := source == "local"
+		useUser := source == "user"
+		usePublic := source == "public"
+
+		sources, _ := findRecipeSourcesByType(useLocal, useUser, usePublic)
+
+		for _, sourceFile := range sources {
+			config, err := loadConfig(sourceFile)
+			if err != nil {
+				continue
+			}
+
+			for _, recipe := range config.Recipes {
+				if recipe.Name == recipeName {
+					return sourceFile, nil
+				}
+
+				if category != "" {
+					combinedName := fmt.Sprintf("%s-%s", category, recipeName)
+					if recipe.Name == combinedName {
+						return sourceFile, nil
+					}
+				}
+			}
+		}
+	}
+
+	return "", fmt.Errorf("recipe not found: %s", recipeName)
+}
+
+func FormatText(text string, color Color, style Style) string {
+	if os.Getenv("NO_COLOR") != "" {
+		return text
+	}
+
+	var result string = text
+
+	if color != ColorNone {
+		if code, ok := colorCodes[string(color)]; ok {
+			result = code + result + colorCodes["reset"]
+		}
+	}
+
+	if style != StyleNone {
+		if color != ColorNone {
+			pos := strings.Index(result, text)
+			if pos != -1 {
+				if code, ok := styleCodes[string(style)]; ok {
+					result = result[:pos] + code + result[pos:]
+					resetPos := strings.LastIndex(result, colorCodes["reset"])
+					if resetPos != -1 {
+						result = result[:resetPos] + styleCodes["reset"] + result[resetPos:]
+					}
+				}
+			}
+		} else {
+			if code, ok := styleCodes[string(style)]; ok {
+				result = code + result + styleCodes["reset"]
+			}
+		}
+	}
+
+	return result
+}
